# Copyright 2020 Catalyst Cloud Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Add quota table

Revision ID: 007
Revises: 006
Create Date: 2020-01-28
"""

# revision identifiers, used by Alembic.
revision = '007'
down_revision = '006'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'quota',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('resource', sa.String(length=50), nullable=False),
        sa.Column('project_id', sa.String(length=128), nullable=False),
        sa.Column('limit', sa.Integer, nullable=False),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('project_id', 'resource'),
        sa.Index(
            'ix_quota_project_id_resource',
            'project_id', 'resource'
        )
    )
